﻿using System;
using System.Linq;

namespace Centido.Management.ConsoleManager
{
    /// <summary>
    ///   A console command that lists all business units from the current domain.
    /// </summary>
    internal class ListUnitsCommand : Command
    {
        /// <summary>
        ///   The display all option parameter.
        /// </summary>
        public const string DisplayAllOptionParameter = "all";

        /// <summary>
        ///   The display all option parameter (short name).
        /// </summary>
        public const string DisplayAllOptionShortParameter = "i";


        /// <summary>
        ///   Gets the console command name.
        /// </summary>
        public override string Name
        {
            get { return "listunits"; }
        }


        /// <summary>
        ///   Gets the console command description.
        /// </summary>
        public override string Description
        {
            get { return Messages.ListUnitsCommandDescription; }
        }

        
        /// <summary>
        ///   Gets the console command full description.
        /// </summary>
        public override string FullDescription
        {
            get { return Messages.ListUnitsCommandFullDescription; }
        }


        /// <summary>
        ///   Executes the console command.
        /// </summary>
        /// <param name="context">The console command execution context.</param>
        /// <exception cref="ArgumentNullException">Thrown when the value of <paramref name="context"/> is null.</exception>
        public override void Execute(CommandExecutionContext context)
        {
            var domain = context.CurrentDomain;

            if (domain == null) return;

            // Enumerating all units
            foreach (var unit in domain.Units.OrderBy(u => u.Name ?? string.Empty))
            {
                // If the display all option is set
                if (context.GetParameterByName(DisplayAllOptionParameter, DisplayAllOptionShortParameter) != null)
                {
                    // If the display name is set
                    if (!string.IsNullOrEmpty(unit.DisplayName))
                        Console.WriteLine("{0} ({1})",
                            unit.Name ?? unit.Id.ToString(),
                            unit.DisplayName);
                    else
                        Console.WriteLine(unit.Name);
                }
                else
                    Console.WriteLine(unit.Name);
            }

            Console.WriteLine();
        }
    }
}
