﻿using System.Collections.Generic;
using Centido.Core.Dto;
using Microsoft.VisualStudio.TestTools.UnitTesting;

namespace Centido.Core.Test
{
    /// <summary>
    ///   Testing the data transfer object for an integer column.
    /// </summary>
    [TestClass]
    public class IntColumnDtoTest : GenericDtoTest
    {
        /// <summary>
        ///   The current domain name.
        /// </summary>
        public const string DomainName = "FullAdminRights";

        /// <summary>
        ///   The current domain.
        /// </summary>
        private Domain _domain;

        /// <summary>
        ///   The table.
        /// </summary>
        private Table _table;

        /// <summary>
        ///   The integer column.
        /// </summary>
        private IntColumn _intColumn;

        /// <summary>
        /// Test initialiaze
        /// </summary>
        [TestInitialize]
        public void Initialize()
        {
            _domain = Domain.Build(DomainConfiguration.Load(DomainName));

            _domain.DeleteAll();

            _table = _domain.AddTable(StringProperty.RandomIdentifier());

            _intColumn = IntColumn.Add(_table, StringProperty.RandomIdentifier());
        }

        /// <summary>
        /// Test clean up
        /// </summary>
        [TestCleanup]
        public void CleanUp()
        {
            _domain.DeleteAll();
        }


        /// <summary>
        ///   Retrieves the name of the data transfer object xml tag.
        /// </summary>
        /// <returns></returns>
        protected override string GetXmlTag()
        {
            return IntColumnDto.XmlTag;
        }


        /// <summary>
        ///   Retrives the current xml schema version.
        /// </summary>
        /// <returns></returns>
        protected override string GetCurrentXmlSchemaVersion()
        {
            return IntColumnDto.CurrentXmlSchemaVersion;
        }


        /// <summary>
        ///   Creates a new data transfer object.
        /// </summary>
        /// <returns></returns>
        protected override DataTransferObject CreateDto()
        {
            return new IntColumnDto
            {
                Name = StringProperty.RandomIdentifier().ToLower(),
                DisplayName = StringProperty.StandardValue('a'),
                Description = StringProperty.ExtendedValue('a'),
                OrderIndex = 11,
                IsNotNull = true,
                DefaultValue = 676,
                Maximum = 78,
                Minimum = 55
            };
        }


        /// <summary>
        ///   Retrieves the list of data transfer object fields.
        /// </summary>
        /// <returns></returns>
        protected override IEnumerable<DtoFieldInfo> GetFields()
        {
            yield return new DtoFieldInfo
            {
                Name = "Name",
                XmlName = ColumnDto.NameAttribute,
                IsRequired = true,
                FieldType = DtoFieldType.String,
                SampleValue = StringProperty.RandomIdentifier()
            };
            yield return new DtoFieldInfo
            {
                Name = "DisplayName",
                XmlName = ColumnDto.DisplayNameAttribute,
                IsRequired = false,
                FieldType = DtoFieldType.String,
                SampleValue = StringProperty.StandardValue('a')
            };
            yield return new DtoFieldInfo
            {
                Name = "Description",
                XmlName = ColumnDto.DescriptionAttribute,
                IsRequired = false,
                FieldType = DtoFieldType.String,
                SampleValue = StringProperty.ExtendedValue('a')
            };
            yield return new DtoFieldInfo
            {
                Name = "OrderIndex",
                XmlName = ColumnDto.OrderIndexAttribute,
                IsRequired = false,
                FieldType = DtoFieldType.Integer,
                SampleValue = 7
            };
            yield return new DtoFieldInfo
            {
                Name = "IsNotNull",
                XmlName = ColumnDto.IsNotNullAttribute,
                IsRequired = false,
                FieldType = DtoFieldType.Boolean,
                SampleValue = true
            };
            yield return new DtoFieldInfo
            {
                Name = "DefaultValue",
                XmlName = IntColumnDto.DefaultValueAttribute,
                IsRequired = false,
                FieldType = DtoFieldType.Integer,
                SampleValue = 5576
            };
            yield return new DtoFieldInfo
            {
                Name = "Maximum",
                XmlName = IntColumnDto.MaximumAttribute,
                IsRequired = false,
                FieldType = DtoFieldType.Integer,
                SampleValue = 678
            };
            yield return new DtoFieldInfo
            {
                Name = "Minimum",
                XmlName = IntColumnDto.MinimumAttribute,
                IsRequired = false,
                FieldType = DtoFieldType.Integer,
                SampleValue = -3478
            };
        }


        /// <summary>
        ///   Copies the properties of the data transfer object to the corresponding object.
        /// </summary>
        /// <param name="dto">The data transfer object.</param>
        /// <returns>Returns the corresponding object after copying.</returns>
        protected override object CopyFrom(DataTransferObject dto)
        {
            ((IntColumnDto)dto).CopyTo(_intColumn);

            return _intColumn;
        }

        /// <summary>
        ///   Creates a data transfer object from the correpsonding object.
        /// </summary>
        /// <param name="dto">The created data transfer object.</param>
        /// <param name="initialObject">The initial object.</param>
        protected override void CreateDtoFromObject(out DataTransferObject dto, out object initialObject)
        {
            dto = IntColumnDto.CreateFrom(_intColumn);
            initialObject = _intColumn;
        }
    }
}
