﻿using System.Collections.Generic;
using Centido.Core.Dto;
using Microsoft.VisualStudio.TestTools.UnitTesting;

namespace Centido.Core.Test
{
    /// <summary>
    ///   Testing the data transfer object for a table permission.
    /// </summary>
    [TestClass]
    public class TablePermissionDtoTest : GenericDtoTest
    {
        /// <summary>
        ///   The current domain name.
        /// </summary>
        public const string DomainName = "FullAdminRights";

        /// <summary>
        ///   The current domain.
        /// </summary>
        private Domain _domain;

        /// <summary>
        ///   The table.
        /// </summary>
        private Table _table;


        /// <summary>
        /// Test initialiaze
        /// </summary>
        [TestInitialize]
        public void Initialize()
        {
            _domain = Domain.Build(DomainConfiguration.Load(DomainName));

            _domain.DeleteAll();

            _table = _domain.AddTable(StringProperty.RandomIdentifier());
        }

        /// <summary>
        /// Test clean up
        /// </summary>
        [TestCleanup]
        public void CleanUp()
        {
            _domain.DeleteAll();
        }


        /// <summary>
        ///   Retrieves the name of the data transfer object xml tag.
        /// </summary>
        /// <returns></returns>
        protected override string GetXmlTag()
        {
            return TablePermissionDto.XmlTag;
        }


        /// <summary>
        ///   Retrives the current xml schema version.
        /// </summary>
        /// <returns></returns>
        protected override string GetCurrentXmlSchemaVersion()
        {
            return TablePermissionDto.CurrentXmlSchemaVersion;
        }


        /// <summary>
        ///   Creates a new data transfer object.
        /// </summary>
        /// <returns></returns>
        protected override DataTransferObject CreateDto()
        {
            return new TablePermissionDto
            {
                DataAction = DataAction.Update,
                AccessLevel = AccessLevel.ChildUnits,
                PrincipalName = StringProperty.RandomIdentifier()
            };
        }


        /// <summary>
        ///   Retrieves the list of data transfer object fields.
        /// </summary>
        /// <returns></returns>
        protected override IEnumerable<DtoFieldInfo> GetFields()
        {
            yield return new DtoFieldInfo
            {
                Name = "DataAction",
                XmlName = TablePermissionDto.DataActionAttribute,
                IsRequired = true,
                FieldType = DtoFieldType.Enumeration,
                SampleValue = (int)DataAction.Delete
            };

            yield return new DtoFieldInfo
            {
                Name = "AccessLevel",
                XmlName = TablePermissionDto.AccessLevelAttribute,
                IsRequired = true,
                FieldType = DtoFieldType.Enumeration,
                SampleValue = (int)AccessLevel.ChildAndParentUnits
            };

            yield return new DtoFieldInfo
            {
                Name = "PrincipalName",
                XmlName = TablePermissionDto.PrincipalNameAttribute,
                IsRequired = true,
                FieldType = DtoFieldType.String,
                SampleValue = "principal_name",
                HasTheSameNameAsCorrespondingObject = false
            };
        }


        /// <summary>
        ///   Copies the properties of the data transfer object to the corresponding object.
        /// </summary>
        /// <param name="dto">The data transfer object.</param>
        /// <returns>Returns the corresponding object after copying.</returns>
        protected override object CopyFrom(DataTransferObject dto)
        {
            return null;
        }


        /// <summary>
        ///   Creates a data transfer object from the correpsonding object.
        /// </summary>
        /// <param name="dto">The created data transfer object.</param>
        /// <param name="initialObject">The initial object.</param>
        protected override void CreateDtoFromObject(out DataTransferObject dto, out object initialObject)
        {
            initialObject = _table.Grant(DataAction.Insert, AccessLevel.ParentUnits);

            dto = TablePermissionDto.CreateFrom(initialObject as TablePermission);
        }
    }
}
