﻿// -----------------------------------------------------------------------
// <copyright file="DbOperationException.cs" company="JSC Centido">
//  Copyright (c) JSC Centido. All rights reserved.
// </copyright>
// -----------------------------------------------------------------------

namespace Centido.Core
{
    using System;
    using System.Data.Common;
    using System.Diagnostics.CodeAnalysis;
    using System.Runtime.Serialization;

    /// <summary>
    ///   The exception that is generated when a database operation failed.
    /// </summary>
    [SuppressMessage("Microsoft.Naming", "CA1709:IdentifiersShouldBeCasedCorrectly", MessageId = "Db"), Serializable]
    public class DbOperationException : DbException
    {
        /// <summary>
        ///   Initializes a new instance of the <see cref = "DbOperationException" /> class.
        /// </summary>
        public DbOperationException()
        {
        }

        /// <summary>
        ///   Initializes a new instance of the <see cref = "DbOperationException" />
        ///   class with a specified error message.
        /// </summary>
        /// <param name = "message">The message that describes the error.</param>
        public DbOperationException(string message)
            : base(message)
        {
        }

        /// <summary>
        ///   Initializes a new instance of the <see cref = "DbOperationException" /> class with a specified error message and a reference to the inner exception that is the cause of this exception.
        /// </summary>
        /// <param name = "message">The error message that explains the reason for the exception.</param>
        /// <param name = "innerException">The exception that is the cause of the current exception, or a null reference (Nothing in Visual Basic) if no inner exception is specified.</param>
        public DbOperationException(string message, Exception innerException)
            : base(message, innerException)
        {
        }

        /// <summary>
        ///   Initializes a new instance of the <see cref = "DbOperationException" />
        ///   class with serialized data.
        /// </summary>
        /// <param name = "info">The <see cref = "System.Runtime.Serialization.SerializationInfo" /> that holds the serialized object data about the exception being thrown. </param>
        /// <param name = "context">The <see cref = "System.Runtime.Serialization.StreamingContext" /> that contains contextual information about the source or destination.</param>
        protected DbOperationException(SerializationInfo info, StreamingContext context)
            : base(info, context)
        {
        }
    }
}