﻿// -----------------------------------------------------------------------
// <copyright file="TableAccessRight.cs" company="JSC Centido">
//  Copyright (c) JSC Centido. All rights reserved.
// </copyright>
// -----------------------------------------------------------------------

namespace Centido.Core
{
    using System;

    /// <summary>
    ///   Table security unit-level access right
    /// </summary>
    public sealed class TableUnitAccessRight : ITableAccessRight
    {
        /// <summary>
        ///   The access type
        /// </summary>
        private readonly DataAction _dataAction;

        /// <summary>
        ///   The target unit id.
        /// </summary>
        private readonly Guid _unitId;

        /// <summary>
        ///   The target table.
        /// </summary>
        private readonly Table _table;

        /// <summary>
        ///   The target user.
        /// </summary>
        private readonly User _user;


        /// <summary>
        ///   Creates an instance of the <see cref="Centido.Core.TableUnitAccessRight"/> class.
        /// </summary>
        /// <param name = "table">The target table.</param>
        /// <param name = "user">The target user.</param>
        /// <param name = "unitId">The target unit id.</param>
        /// <param name = "dataAction">The access type</param>
        /// <exception cref="ArgumentNullException">Thrown when the value of <paramref name="table"/> is null.</exception>
        /// <exception cref="ArgumentNullException">Thrown when the value of <paramref name="user"/> is null.</exception>
        internal TableUnitAccessRight(Table table, User user, Guid unitId, DataAction dataAction)
        {
            if (table == null)
                throw new ArgumentNullException("table");
            if (user == null)
                throw new ArgumentNullException("user");

            _table = table;
            _user = user;
            _unitId = unitId;
            _dataAction = dataAction;
        }

        /// <summary>
        ///   Gets the taret unit id.
        /// </summary>
        public Guid UnitId
        {
            get { return _unitId; }
        }

        #region ITableAccessRight Members

        /// <summary>
        ///   Gets the target table.
        /// </summary>
        public Table Table
        {
            get { return _table; }
        }

        /// <summary>
        ///   Gets the target user.
        /// </summary>
        public User User
        {
            get { return _user; }
        }

        /// <summary>
        ///   Gets the data action.
        /// </summary>
        public DataAction DataAction
        {
            get { return _dataAction; }
        }

        #endregion
    }
}