﻿using System;
using System.Linq;

namespace Centido.Management.ConsoleManager
{
    /// <summary>
    ///   A console command that lists all table in the current domain.
    /// </summary>
    internal class ListTablesCommand : Command
    {
        /// <summary>
        ///   The display all option parameter.
        /// </summary>
        public const string DisplayAllOptionParameter = "all";

        /// <summary>
        ///   The display all option parameter (short name).
        /// </summary>
        public const string DisplayAllOptionShortParameter = "i";


        /// <summary>
        ///   Gets the console command name.
        /// </summary>
        public override string Name
        {
            get { return "listtables"; }
        }


        /// <summary>
        ///   Gets the console command description.
        /// </summary>
        public override string Description
        {
            get { return Messages.ListTablesCommandDescription; }
        }

        
        /// <summary>
        ///   Gets the console command full description.
        /// </summary>
        public override string FullDescription
        {
            get { return Messages.ListTablesCommandFullDescription; }
        }


        /// <summary>
        ///   Executes the console command.
        /// </summary>
        /// <param name="context">The console command execution context.</param>
        /// <exception cref="ArgumentNullException">Thrown when the value of <paramref name="context"/> is null.</exception>
        public override void Execute(CommandExecutionContext context)
        {
            var domain = context.CurrentDomain;

            if (domain == null) return;

            // Enumerating all tables
            foreach (var table in domain.Tables.OrderBy(t => t.Name))
            {
                // If the display all option is set
                if (context.GetParameterByName(DisplayAllOptionParameter, DisplayAllOptionShortParameter) != null)
                {
                    // If the display name is set
                    if (!string.IsNullOrEmpty(table.DisplayName))
                        Console.WriteLine("{0} ({1}) {2}",
                            table.Name,
                            table.DisplayName,
                            table.Description ?? string.Empty);
                    else
                        Console.WriteLine(table.Name);
                }
                else
                    Console.WriteLine(table.Name);
            }

            Console.WriteLine();
        }
    }
}
